# -*- coding: utf-8 -*-
# Generated by Django 1.11.20 on 2019-05-09 21:58

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('coop_cms', '0015_auto_20170608_1600'),
    ]

    operations = [
        migrations.AlterField(
            model_name='articlecategory',
            name='sites',
            field=models.ManyToManyField(default=[], to='sites.Site', verbose_name='site'),
        ),
        migrations.AlterField(
            model_name='newsletter',
            name='site',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='sites.Site', verbose_name='site'),
        ),
    ]
