# Generated by Django 2.0.13 on 2020-01-24 13:41

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('coop_cms', '0016_auto_20190509_2158'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='document',
            options={'ordering': ('ordering',), 'verbose_name': 'document', 'verbose_name_plural': 'documents'},
        ),
        migrations.AlterModelOptions(
            name='image',
            options={'ordering': ('ordering',), 'verbose_name': 'image', 'verbose_name_plural': 'images'},
        ),
    ]
