{% extends "coop_cms/base.html" %}
{% load i18n static coop_edition %}

{% block meta_title %}
    {{ category.name }}
{% endblock %}

{% block document %}
    {% block title %}
        <h2 id="title">{{ category.name }}</h2>
        Keep this for unit-testing: This comes from custom template
    {% endblock %}
    
    {% for item in articles %}
        {% include "coop_cms/article_list_item.html" %}
    {% endfor %}
    
    {% include "coop_cms/_pagination.html" with page=articles pages=pages %}

{% endblock %}
